<?php

declare(strict_types=1);

namespace Pdahal\Xmpp\Loggers;

use Exception;

class Logger implements Loggable
{
    public mixed $log;

    public const LOG_FOLDER = "logs";
    public const LOG_FILE = "xmpp.log";

    public function __construct()
    {
        $this->createLogFile();
        $this->log = fopen(self::LOG_FOLDER . '/' . self::LOG_FILE, 'a');
    }

    protected function createLogFile(): void
    {
        if (!file_exists(self::LOG_FOLDER)) {
            mkdir(self::LOG_FOLDER, 0777, true);
        }
    }

    public function log(string $message): void
    {
        $this->writeToLog($message);
    }

    public function logRequest(string $message): void
    {
        $this->writeToLog($message, "REQUEST");
    }

    public function logResponse(string $message): void
    {
        $this->writeToLog($message, "RESPONSE");
    }

    public function error(string $message): void
    {
        $this->writeToLog($message, "ERROR");
    }

    protected function writeToLog(string $message, string $type = ''): void
    {
        $prefix = date("Y.m.d H:i:s") . " " . session_id() . ($type ? " {$type}::" : " ");
        $this->writeToFile($this->log, $prefix . "$message\n");
    }

    protected function writeToFile(mixed $file, string $message): void
    {
        try {
            fwrite($file, $message);
        } catch (Exception) {
            // silent fail
        }
    }

    public function getFilePathFromResource(mixed $resource): string
    {
        $metaData = stream_get_meta_data($resource);
        return $metaData["uri"];
    }
}
